// CRR3Engine.h - give us a nice API for sending device commands
///\file CRR3Engine.h
///\author David Phillip Oster, December 2003
///\brief give us a nice API for sending device commands
/*
This software is provided 'as-is', without any express or implied   warranty.  In no event will the authors be held liable for any damages   arising from the use of this software.

Permission is granted to anyone to use this software for any purpose,   including commercial applications, and to alter it and redistribute it   freely, subject to the following restrictions:

1. The origin of this software must not be misrepresented; you must not      claim that you wrote the original software. If you use this software in a product, an acknowledgment in the product documentation would be      appreciated but is not required.

2. Altered source versions must be plainly marked as such, and must not be      misrepresented as being the original software.

3. This notice may not be removed or altered from any source distribution.
	David Phillip Oster <mailto:oster@ieee.org> Dec 15, 2003
 */
#pragma once

#include <utility>

/// give us a nice API for sending device commands
class CRR3Engine {
public:
	typedef void (*ReporterFunc)(LPCTSTR  /*inMesg*/, void * /*envion*/);
private:	// data members
	IRedRatLookup* m_rrLookup;
	ICCWRedRat3*   m_rr3;
	ISignalDB*	   m_signalDB;
	CString		   m_dbPath;
	ReporterFunc   m_ReporterFunc;
	void*		   m_ReporterEnviron;

public:
	/// pass in the path to the db.xml file to load in.
	CRR3Engine(LPCTSTR dbPath);
	~CRR3Engine();

	/// do the command.
	void IREmit(LPCTSTR devName, LPCTSTR  keyName);

	/// true if the device exists in the database. Never throws.
	bool IRExists(LPCTSTR devName);

	/// true if command for this device exists in the database. Never throws
	bool IRExists(LPCTSTR devName, LPCTSTR  keyName);

	/// blink the visible LED on the device.
	void Blink();

	/// allow the client to install a different function to report errors to the user.
	/// the default does a message box. Pass NULL, NULL to restore the default.
	/// \return previous value, so you can save and restore it. It is safe to ignore the return value.
	std::pair<ReporterFunc, void*> SetErrorReporter(ReporterFunc inFunc, void *environ);

private:
	void	Init();						// construction helper
	void	Die();						// destruction helper.

	CRR3Engine(const CRR3Engine&);		// not copyable
	CRR3Engine& operator =(const CRR3Engine&);
};

