// CVCRAction.cpp - defines a VCR action: for now, record channel XXX or: stop.
/*
This software is provided 'as-is', without any express or implied   warranty.  In no event will the authors be held liable for any damages   arising from the use of this software.

Permission is granted to anyone to use this software for any purpose,   including commercial applications, and to alter it and redistribute it   freely, subject to the following restrictions:

1. The origin of this software must not be misrepresented; you must not      claim that you wrote the original software. If you use this software in a product, an acknowledgment in the product documentation would be      appreciated but is not required.

2. Altered source versions must be plainly marked as such, and must not be      misrepresented as being the original software.

3. This notice may not be removed or altered from any source distribution.
	David Phillip Oster <mailto:oster@ieee.org> Dec 15, 2003
 */
#include "StdAfx.h"
#include "CVCRAction.h"
#include "CRR3Engine.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

CVCRStartAction::CVCRStartAction(CRR3Engine *inEngine, int vcr, int channel){
	ASSERT(NULL != m_Engine);
	ASSERT(1 <= vcr && vcr <= 4);
	ASSERT(1 <= channel && channel <= 999);
	m_Engine = inEngine;
	m_vcr = vcr;
	m_channel = channel;
}

namespace{

/// vcr is from 1..4
LPCTSTR GetVCRName(int vcr){
	LPCTSTR vcrNames[] = {
		_T(""),
		_T("VCR 1"),
		_T("VCR 2"),
		_T("VCR 3"),
		_T("VCR 4")
	};
	return vcrNames[vcr];
}

} // end anonymous namespace

void CVCRStartAction::Do(){
	m_Engine->IREmit(GetVCRName(m_vcr), _T("Stop"));
	m_Engine->Blink();
	CString channelString;
	channelString.Format(_T("%d"), m_channel);
	int iMax = channelString.GetLength();
	if(1 <= iMax){
		m_Engine->IREmit(GetVCRName(m_vcr), _T("0"));
		m_Engine->Blink();
	}

	// in our setup, only VCR 2 really would like a 3 digit channel number.
	if(2 <= iMax && 2 == m_vcr){
		m_Engine->IREmit(GetVCRName(m_vcr), _T("0"));
		m_Engine->Blink();
	}

	for(int i = 0; i < iMax; ++i){
		CString digit;
		digit.Format(_T("%c"), channelString[i]);
		m_Engine->IREmit(GetVCRName(m_vcr), digit);
		m_Engine->Blink();
	}
	m_Engine->IREmit(GetVCRName(m_vcr), _T("Record"));
	m_Engine->Blink();
}

CVCRStopAction::CVCRStopAction(CRR3Engine *inEngine, int vcr){
	ASSERT(NULL != m_Engine);
	ASSERT(1 <= vcr && vcr <= 4);
	m_Engine = inEngine;
	m_vcr = vcr;
}

void CVCRStopAction::Do(){
	m_Engine->IREmit(GetVCRName(m_vcr), _T("Stop"));
	m_Engine->Blink();
}
