// CVCRAction.h - send a complete command to the VCR now. Currently, only commands are
///\file CVCRAction.h
///\author David Phillip Oster
///\brief send a complete command to the VCR now. 
///
/// Represent a complete command to the VCR. Currently, the only commands are CVCRStartAction and CVCRStopAction.
/// They have a common base class, so you can have an array of pointers to them to do.
/// They actually do the command, when you call their Do() member.

/*
This software is provided 'as-is', without any express or implied warranty. In no event will the authors be held liable for any damages arising from the use of this software.

Permission is granted to anyone to use this software for any purpose, including commercial applications, and to alter it and redistribute it freely, subject to the following restrictions:

1. The origin of this software must not be misrepresented; you must not claim that you wrote the original software. If you use this software in a product, an acknowledgment in the product documentation would be appreciated but is not required.

2. Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

3. This notice may not be removed or altered from any source distribution.
	David Phillip Oster <mailto:oster@ieee.org> Dec 15, 2003
 */

#pragma once

class CRR3Engine;

class CVCRAction {
public:
			CVCRAction(){}
	virtual ~CVCRAction(){}
	virtual void Do()=0;
};

class CVCRStartAction : public CVCRAction{
private:
	CRR3Engine *m_Engine;	// does not own
	int		m_vcr;			// ones based: u.e, 1 == vcr 1
	int		m_channel;
public:
				CVCRStartAction(CRR3Engine *inEngine, int vcr, int channel);
	virtual void Do();
};

class CVCRStopAction : public CVCRAction{
private:
	CRR3Engine *m_Engine;	// does not own
	int		m_vcr;			// ones based: u.e, 1 == vcr 1
public:
				CVCRStopAction(CRR3Engine *inEngine, int vcr);
	virtual void Do();
};