// RR1Dlg.cpp : implementation file
/*
This software is provided 'as-is', without any express or implied warranty. In no event will the authors be held liable for any damages arising from the use of this software.

Permission is granted to anyone to use this software for any purpose, including commercial applications, and to alter it and redistribute it freely, subject to the following restrictions:

1. The origin of this software must not be misrepresented; you must not claim that you wrote the original software. If you use this software in a product, an acknowledgment in the product documentation would be appreciated but is not required.

2. Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.

3. This notice may not be removed or altered from any source distribution.
	David Phillip Oster <mailto:oster@ieee.org> Dec 15, 2003
 */

#include "stdafx.h"
#include "RR1.h"
#include "RR1Dlg.h"
#include "CAboutDlg.h"
#include "CRR3Engine.h"
#include "DlgProxy.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

const char *sDBName = "DeviceSignalDatabase.xml";
const int kNumVCRs = 4;

/////////////////////////////////////////////////////////////////////////////
// CRR1Dlg dialog

IMPLEMENT_DYNAMIC(CRR1Dlg, CDialog);

CRR1Dlg::CRR1Dlg(CWnd* pParent /*=NULL*/)
	: CDialog(CRR1Dlg::IDD, pParent), 
	m_Engine(new CRR3Engine(sDBName)){	// "K:\\David's\\RedRat\\DeviceSignalDatabase.xml"

	//{{AFX_DATA_INIT(CRR1Dlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
	m_pAutoProxy = NULL;

	m_IsInitialUpdate = true;	// for the first one.
	m_vcrN = 1;	// i.e., VCR2 (we are zerobased inside the program)
}

CRR1Dlg::~CRR1Dlg(){
	// If there is an automation proxy for this dialog, set
	// its back pointer to this dialog to NULL, so it knows
	// the dialog has been deleted.
	if (m_pAutoProxy != NULL){
		m_pAutoProxy->m_pDialog = NULL;
	}
}

void CRR1Dlg::DoDataExchange(CDataExchange* pDX){
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CRR1Dlg)
	DDX_Control(pDX, IDC_TAB1, m_VCR);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CRR1Dlg, CDialog)
	//{{AFX_MSG_MAP(CRR1Dlg)
	ON_WM_SYSCOMMAND()
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_WM_CLOSE()
	ON_BN_CLICKED(IDC_0, On0)
	ON_BN_CLICKED(IDC_1, On1)
	ON_BN_CLICKED(IDC_2, On2)
	ON_BN_CLICKED(IDC_3, On3)
	ON_BN_CLICKED(IDC_4, On4)
	ON_BN_CLICKED(IDC_5, On5)
	ON_BN_CLICKED(IDC_6, On6)
	ON_BN_CLICKED(IDC_7, On7)
	ON_BN_CLICKED(IDC_8, On8)
	ON_BN_CLICKED(IDC_9, On9)
	ON_BN_CLICKED(IDC_RECORD, OnRecord)
	ON_BN_CLICKED(IDC_STOP, OnStop)
	ON_NOTIFY(NM_CLICK, IDC_TAB1, OnClickTab1)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CRR1Dlg message handlers

BOOL CRR1Dlg::OnInitDialog(){
	CDialog::OnInitDialog();

	// Add "About..." menu item to system menu.

	// IDM_ABOUTBOX must be in the system command range.
	ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
	ASSERT(IDM_ABOUTBOX < 0xF000);

	CMenu* pSysMenu = GetSystemMenu(FALSE);
	if (pSysMenu != NULL){
		CString strAboutMenu;
		strAboutMenu.LoadString(IDS_ABOUTBOX);
		if( ! strAboutMenu.IsEmpty()){
			pSysMenu->AppendMenu(MF_SEPARATOR);
			pSysMenu->AppendMenu(MF_STRING, IDM_ABOUTBOX, strAboutMenu);
		}
	}

	// Set the icon for this dialog. The framework does this automatically
	// when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
	// TODO: Add extra initialization here
	InitVCRTabControl();
	ValidateEngine();

	return TRUE; // return TRUE unless you set the focus to a control
}

// 
void CRR1Dlg::DoInitialUpdate(){
}

#define COUNTOF(a) (sizeof(a)/sizeof(*(a)))

const char *commands[] = {
	"0",
	"1",
	"2",
	"3",
	"4",
	"5",
	"6",
	"7",
	"8",
	"9",
	"Stop",
	"Record"
};

void CRR1Dlg::ValidateEngine(){
	int i;
	int errNum = IDSS_MISSINGDBDEV;
	CString s;
	for(i = 0;i < kNumVCRs; ++i){
		if( ! m_Engine->IRExists(GetVCRName(i))){
			if(0 != s.GetLength()){
				s += ", ";
				errNum = IDSS_MISSINGDBDEVS;
			}
			s += GetVCRName(i);
		}
	}
	if(0 != s.GetLength()){
		CString mesg;
		mesg.Format(errNum, static_cast<const char *>(sDBName), static_cast<const char *>(s));
		AfxMessageBox(mesg, MB_ICONSTOP);
		AfxThrowUserException();
	}
	errNum = IDSS_MISSINGDBCOMMAND;
	for(i = 0;i < kNumVCRs; ++i){
		for(int j = 0; j < COUNTOF(commands); ++j){
			if( ! m_Engine->IRExists(GetVCRName(i), commands[j])){
				CString s1;
				s1.Format("%s: \"%s\"", GetVCRName(i), commands[j]);
				if(0 != s.GetLength()){
					s += ", ";
					errNum = IDSS_MISSINGDBCOMMAND;
				}
				s += s1;
			}
		}
	}
	if(0 != s.GetLength()){
		CString mesg;
		mesg.Format(errNum, static_cast<const char *>(sDBName), static_cast<const char *>(s));
		AfxMessageBox(mesg, MB_ICONSTOP);
		AfxThrowUserException();
	}
}

/// initialize our tabs.
void CRR1Dlg::InitVCRTabControl(){
	m_VCR.SetPadding(5);
	m_VCR.SetMinTabWidth(2);
	for(int i = 0; i < kNumVCRs; ++i){
		CString s;
		s.Format("VCR%d", i+1);
		m_VCR.InsertItem(i, s);
	}
	m_VCR.SetCurSel(m_vcrN);
}


void CRR1Dlg::OnSysCommand(UINT nID, LPARAM lParam){
	switch((0xFFF0 & nID)){
	case IDM_ABOUTBOX: 
		{
		CAboutDlg dlgAbout;
		dlgAbout.DoModal();
		break;
		}
	default:
		{
		CDialog::OnSysCommand(nID, lParam);
		break;
		}
	}
}

// If you add a minimize button to your dialog, you will need the code below
// to draw the icon. For MFC applications using the document/view model,
// this is automatically done for you by the framework.

void CRR1Dlg::OnPaint() {
	if (IsIconic()){
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}else{
		if(m_IsInitialUpdate){
			DoInitialUpdate();	// we are just about to paint, so we do final initializations
			m_IsInitialUpdate = false;
		}
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
// the minimized window.
HCURSOR CRR1Dlg::OnQueryDragIcon(){
	return (HCURSOR) m_hIcon;
}

// Automation servers should not exit when a user closes the UI
// if a controller still holds on to one of its objects. These
// message handlers make sure that if the proxy is still in use,
// then the UI is hidden but the dialog remains around if it
// is dismissed.

void CRR1Dlg::OnClose(){
	if (CanExit()){
		CDialog::OnClose();
	}
}

void CRR1Dlg::OnOK(){
	if (CanExit()){
		CDialog::OnOK();
	}
}

void CRR1Dlg::OnCancel(){
	if (CanExit()){
		CDialog::OnCancel();
	}
}

// If the proxy object is still around, then the automation
// controller is still holding on to this application. Leave
// the dialog around, but hide its UI.
BOOL CRR1Dlg::CanExit(){
	if (m_pAutoProxy != NULL)
	{
		ShowWindow(SW_HIDE);
		return FALSE;
	}

	return TRUE;
}

void CRR1Dlg::On0() { DoIRButton("0"); }
void CRR1Dlg::On1() { DoIRButton("1"); }
void CRR1Dlg::On2() { DoIRButton("2"); }
void CRR1Dlg::On3(){ DoIRButton("3"); } 
void CRR1Dlg::On4(){ DoIRButton("4"); }
void CRR1Dlg::On5(){ DoIRButton("5"); } 
void CRR1Dlg::On6(){ DoIRButton("6"); }
void CRR1Dlg::On7(){ DoIRButton("7"); }
void CRR1Dlg::On8(){ DoIRButton("8"); }
void CRR1Dlg::On9(){ DoIRButton("9"); }
void CRR1Dlg::OnRecord(){ DoIRButton("Record"); }
void CRR1Dlg::OnStop(){	DoIRButton("Stop"); }

void CRR1Dlg::DoIRButton(const char *key){
	// lookup key in the database for the current device and do it.
	m_Engine->IREmit(GetVCRName(m_vcrN), key);
	m_Engine->Blink();
}


// update our state of which tab is current.
void CRR1Dlg::OnClickTab1(NMHDR* pNMHDR, LRESULT* pResult){
	m_vcrN = m_VCR.GetCurSel();	
	*pResult = 0;
}

// given a tab index, return the name of a corresponding device
// where name means a string appropriate to hand to the engine.
const char* CRR1Dlg::GetVCRName(int index){
	const char *names[] = {
		"VCR 1",
		"VCR 2",
		"VCR 3",
		"VCR 4"
	};
	return names[index];
}
